import { sendJSON, parseBody } from "../utils/helpers.js";
import {
  getTodos,
  addTodo,
  updateTodoById,
  deleteTodoById,
} from "../data/todos.js";

/**
 * @typedef {Object} Todo
 * @property {number} id - Todo unique identifier
 * @property {string} text - Todo text content
 * @property {boolean} completed - Todo completion status
 */

/**
 * @typedef {Object} CreateTodoRequest
 * @property {string} text - Todo text content
 */

/**
 * @typedef {Object} UpdateTodoRequest
 * @property {string} [text] - Todo text content
 * @property {boolean} [completed] - Todo completion status
 */

/**
 * @typedef {Object} RequestWithParams
 * @property {Object} params - Request parameters
 * @property {string} params.id - Todo ID from URL
 */

/**
 * Get all todos
 * @param {import('http').IncomingMessage} req - HTTP request object
 * @param {import('http').ServerResponse} res - HTTP response object
 * @returns {void}
 */
export const getAllTodos = (req, res) => {
  /** @type {Todo[]} */
  const todos = getTodos();
  sendJSON(res, 200, todos);
};

/**
 * Create a new todo
 * @param {import('http').IncomingMessage} req - HTTP request object
 * @param {import('http').ServerResponse} res - HTTP response object
 * @returns {void}
 */
export const createTodo = (req, res) => {
  parseBody(req, (error, data) => {
    if (error) {
      return sendJSON(res, 400, { error: "Invalid JSON" });
    }

    /** @type {CreateTodoRequest} */
    const todoData = data;

    if (!todoData.text) {
      return sendJSON(res, 400, { error: "Text is required" });
    }

    /** @type {Todo} */
    const newTodo = addTodo(todoData.text);
    sendJSON(res, 201, newTodo);
  });
};

/**
 * Update an existing todo
 * @param {import('http').IncomingMessage & RequestWithParams} req - HTTP request object with params
 * @param {import('http').ServerResponse} res - HTTP response object
 * @returns {void}
 */
export const updateTodo = (req, res) => {
  parseBody(req, (error, data) => {
    if (error) {
      return sendJSON(res, 400, { error: "Invalid JSON" });
    }

    /** @type {UpdateTodoRequest} */
    const updateData = data;

    /** @type {Todo | null} */
    const updatedTodo = updateTodoById(req.params.id, updateData);

    if (updatedTodo) {
      sendJSON(res, 200, updatedTodo);
    } else {
      sendJSON(res, 404, { error: "Todo not found" });
    }
  });
};

/**
 * Delete a todo
 * @param {import('http').IncomingMessage & RequestWithParams} req - HTTP request object with params
 * @param {import('http').ServerResponse} res - HTTP response object
 * @returns {void}
 */
export const deleteTodo = (req, res) => {
  /** @type {boolean} */
  const deleted = deleteTodoById(req.params.id);

  if (deleted) {
    sendJSON(res, 200, { message: "Todo deleted" });
  } else {
    sendJSON(res, 404, { error: "Todo not found" });
  }
};
