import http from "node:http";
import url from "node:url";
import { sendJSON, setCorsHeaders } from "./utils/helpers.js";
import {
  getAllTodos,
  createTodo,
  updateTodo,
  deleteTodo,
} from "./controllers/todoController.js";

/**
 * @typedef {Object} ApiEndpoints
 * @property {string} "GET /todos" - Get all todos
 * @property {string} "POST /todos" - Create todo
 * @property {string} "PUT /todos/:id" - Update todo
 * @property {string} "DELETE /todos/:id" - Delete todo
 */

/**
 * @typedef {Object} RootResponse
 * @property {string} message - API name
 * @property {string} version - API version
 * @property {ApiEndpoints} endpoints - Available endpoints
 */

/**
 * @typedef {Object} ErrorResponse
 * @property {string} error - Error message
 */

/**
 * HTTP server that handles Todo API requests
 * @type {http.Server}
 */
const server = http.createServer((req, res) => {
  /** @type {url.UrlWithParsedQuery} */
  const parsedUrl = url.parse(req.url, true);
  /** @type {string} */
  const path = parsedUrl.pathname;
  /** @type {string} */
  const method = req.method;

  setCorsHeaders(res);

  if (method === "OPTIONS") {
    res.statusCode = 200;
    res.end();
    return;
  }

  // Routes
  if (path === "/" && method === "GET") {
    // Root route
    sendJSON(res, 200, {
      message: "Todo API",
      version: "1.0.0",
      endpoints: {
        "GET /todos": "Get all todos",
        "POST /todos": "Create todo",
        "PUT /todos/:id": "Update todo",
        "DELETE /todos/:id": "Delete todo",
      },
    });
  } else if (path === "/todos" && method === "GET") {
    getAllTodos(req, res);
  } else if (path === "/todos" && method === "POST") {
    createTodo(req, res);
  } else if (path.match(/^\/todos\/\d+$/) && method === "PUT") {
    /** @type {string} */
    const id = path.split("/")[2];
    req.params = { id };
    updateTodo(req, res);
  } else if (path.match(/^\/todos\/\d+$/) && method === "DELETE") {
    /** @type {string} */
    const id = path.split("/")[2];
    req.params = { id };
    deleteTodo(req, res);
  } else {
    sendJSON(res, 404, { error: "Route not found" });
  }
});

export default server;
