/**
 * @callback ParseBodyCallback
 * @param {Error | null} error - Parse error if any
 * @param {Object} data - Parsed JSON data
 * @returns {void}
 */

/**
 * Set CORS headers on response
 * @param {import('http').ServerResponse} res - HTTP response object
 * @returns {void}
 */
export const setCorsHeaders = (res) => {
  res.setHeader("Access-Control-Allow-Origin", "*");
  res.setHeader("Access-Control-Allow-Methods", "GET, POST, PUT, DELETE");
  res.setHeader("Access-Control-Allow-Headers", "Content-Type");
};

/**
 * Send JSON response
 * @param {import('http').ServerResponse} res - HTTP response object
 * @param {number} statusCode - HTTP status code
 * @param {Object | Array} data - Data to send as JSON
 * @returns {void}
 */
export const sendJSON = (res, statusCode, data) => {
  res.statusCode = statusCode;
  res.setHeader("Content-Type", "application/json");
  res.end(JSON.stringify(data));
};

/**
 * Parse request body as JSON
 * @param {import('http').IncomingMessage} req - HTTP request object
 * @param {ParseBodyCallback} callback - Callback function with parsed data or error
 * @returns {void}
 */
export const parseBody = (req, callback) => {
  /** @type {string} */
  let body = "";
  req.on("data", (chunk) => {
    body += chunk.toString();
  });
  req.on("end", () => {
    try {
      /** @type {Object} */
      const data = body ? JSON.parse(body) : {};
      callback(null, data);
    } catch (error) {
      callback(error);
    }
  });
};
